@@ -95,24 +95,24 @@ list__process_line (char     *line,
 	g_return_if_fail (line != NULL);
 
 	if (! p7z_comm->list_started) {
-		if (strncmp (line, "7-Zip ", 6) == 0) {
+		if (strncmp (line, "p7zip Version ", 14) == 0) {
 			const char *ver_start;
 			int         ver_len;
-		        char        version[256];
+			char        version[256];
 
-			ver_start = eat_spaces (line + 6);
-		        ver_len = strchr (ver_start, ' ') - ver_start;
-        		strncpy (version, ver_start, ver_len);
-		        version[ver_len] = 0;
+			ver_start = eat_spaces (line + 14);
+			ver_len = strchr (ver_start, ' ') - ver_start;
+			strncpy (version, ver_start, ver_len);
+			version[ver_len] = 0;
 
-			if (strcmp (version, "4.57") < 0)
+			if (strcmp (version, "4.55") < 0)
 				p7z_comm->old_style = TRUE;
 			else
 				p7z_comm->old_style = FALSE;
 		}
 		else if (p7z_comm->old_style && (strncmp (line, "Listing archive: ", 17) == 0))
 			p7z_comm->list_started = TRUE;
-		else if (! p7z_comm->old_style && (strncmp (line, "--------", 8) == 0))
+		else if (! p7z_comm->old_style && (strcmp (line, "----------") == 0))
 			p7z_comm->list_started = TRUE;
 		else if (strncmp (line, "Multivolume = ", 14) == 0) {
 			fields = g_strsplit (line, " = ", 2);
@@ -124,14 +124,20 @@ list__process_line (char     *line,
 
 	if (strcmp (line, "") == 0) {
 		if (p7z_comm->fdata != NULL) {
-			fdata = p7z_comm->fdata;
-			if (fdata->dir)
-				fdata->name = dir_name_from_path (fdata->full_path);
-			else
-				fdata->name = g_strdup (file_name_from_path (fdata->full_path));
-			fdata->path = remove_level_from_path (fdata->full_path);
-			fr_command_add_file (comm, fdata);
-			p7z_comm->fdata = NULL;
+			if (p7z_comm->fdata->original_path == NULL) {
+				file_data_free (p7z_comm->fdata);
+				p7z_comm->fdata = NULL;
+			}
+			else {
+				fdata = p7z_comm->fdata;
+				if (fdata->dir)
+					fdata->name = dir_name_from_path (fdata->full_path);
+				else
+					fdata->name = g_strdup (file_name_from_path (fdata->full_path));
+				fdata->path = remove_level_from_path (fdata->full_path);
+				fr_command_add_file (comm, fdata);
+				p7z_comm->fdata = NULL;
+			}
 		}
 		return;
 	}
@@ -166,7 +172,8 @@ list__process_line (char     *line,
 		char **modified_fields;
 
 		modified_fields = g_strsplit (fields[1], " ", 2);
-		fdata->modified = mktime_from_string (modified_fields[0], modified_fields[1]);
+		if (modified_fields[0] != NULL)
+			fdata->modified = mktime_from_string (modified_fields[0], modified_fields[1]);
 		g_strfreev (modified_fields);
 	}
 	else if (strcmp (fields[0], "Encrypted") == 0) {
@@ -212,16 +219,26 @@ add_password_arg (FrCommand     *comm,
 }
 
 
+static void
+list__begin (gpointer data)
+{
+	FrCommand7z *p7z_comm = data;
+
+	if (p7z_comm->fdata != NULL) {
+		file_data_free (p7z_comm->fdata);
+		p7z_comm->fdata = NULL;
+	}
+	p7z_comm->list_started = FALSE;
+}
+
+
 static void
 fr_command_7z_list (FrCommand  *comm)
 {
-	FrCommand7z *p7z_comm = FR_COMMAND_7Z (comm);
-
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
-				      list__process_line,
-				      comm);
+	fr_process_set_out_line_func (comm->process, list__process_line, comm);
 
 	fr_command_7z_begin_command (comm);
+	fr_process_set_begin_func (comm->process, list__begin, comm);
 	fr_process_add_arg (comm->process, "l");
 	fr_process_add_arg (comm->process, "-slt");
 	fr_process_add_arg (comm->process, "-bd");
@@ -231,11 +248,6 @@ fr_command_7z_list (FrCommand  *comm)
 	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 
-	if (p7z_comm->fdata != NULL) {
-		file_data_free (p7z_comm->fdata);
-		p7z_comm->fdata = NULL;
-	}
-	p7z_comm->list_started = FALSE;
 	fr_process_start (comm->process);
 }
 
@@ -335,7 +347,7 @@ fr_command_7z_add (FrCommand     *comm,
 		fr_process_add_arg (comm->process, "-mx=7"); break;
 	}
 
-	if (is_mime_type (comm->mime_type, "application/x-executable"))
+	if (is_mime_type (comm->mime_type, "application/x-ms-dos-executable"))
 		fr_process_add_arg (comm->process, "-sfx");
 
 	if (comm->volume_size > 0)
@@ -365,7 +377,7 @@ fr_command_7z_delete (FrCommand  *comm,
 	fr_process_add_arg (comm->process, "d");
 	fr_process_add_arg (comm->process, "-bd");
 	fr_process_add_arg (comm->process, "-y");
-	if (is_mime_type (comm->mime_type, "application/x-executable"))
+	if (is_mime_type (comm->mime_type, "application/x-ms-dos-executable"))
 		fr_process_add_arg (comm->process, "-sfx");
 
 	if (from_file != NULL)
@@ -481,7 +493,7 @@ const char *sevenz_mime_types[] = { "application/x-7z-compressed",
 				    "application/x-cd-image",
 				    /*"application/x-cbr",*/
 				    "application/x-cbz",
-				    "application/x-executable",
+				    "application/x-ms-dos-executable",
 				    "application/x-rar",
 				    "application/zip",
 				    NULL };
@@ -496,7 +508,7 @@ fr_command_7z_get_mime_types (FrCommand *comm)
 
 FrCommandCap
 fr_command_7z_get_capabilities (FrCommand  *comm,
-			        const char *mime_type)
+				const char *mime_type)
 {
 	FrCommandCap capabilities;
 
@@ -504,13 +516,24 @@ fr_command_7z_get_capabilities (FrCommand  *comm,
 	if (! is_program_in_path ("7za") && ! is_program_in_path ("7zr") && ! is_program_in_path ("7z"))
 		return capabilities;
 
-	if (is_mime_type (mime_type, "application/x-7z-compressed"))
+	if (is_mime_type (mime_type, "application/x-7z-compressed")) {
 		capabilities |= FR_COMMAND_CAN_READ_WRITE | FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER | FR_COMMAND_CAN_CREATE_VOLUMES;
-
+	}
+	else if (is_mime_type (mime_type, "application/x-7z-compressed-tar")) {
+		capabilities |= FR_COMMAND_CAN_READ_WRITE | FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER | FR_COMMAND_CAN_CREATE_VOLUMES;
+	}
 	else if (is_program_in_path ("7z")) {
-		capabilities |= FR_COMMAND_CAN_READ;
+		if (is_mime_type (mime_type, "application/x-rar")
+		    || is_mime_type (mime_type, "application/x-cbr"))
+		{
+			if (g_file_test ("/usr/lib/p7zip/Codecs/Rar29.so", G_FILE_TEST_EXISTS))
+				capabilities |= FR_COMMAND_CAN_READ;
+		}
+		else
+			capabilities |= FR_COMMAND_CAN_READ;
+
 		if (is_mime_type (mime_type, "application/x-cbz")
-		    || is_mime_type (mime_type, "application/x-executable")
+		    || is_mime_type (mime_type, "application/x-ms-dos-executable")
 		    || is_mime_type (mime_type, "application/zip"))
 		{
 			capabilities |= FR_COMMAND_CAN_WRITE | FR_COMMAND_CAN_ENCRYPT;
