@@ -3,8 +3,8 @@
 
 #include <glib-object.h>
 
-/* enumerations from "typedefs.h" */
-#include "typedefs.h"
+/* enumerations from "../../src/typedefs.h" */
+#include "../../src/typedefs.h"
 GType
 fr_compression_get_type (void)
 {
